#include "../../include/package.h"
#include "../../include/error.h"
#include "../../include/pool.h"
#include "../../include/util.h"
#include "../../include/ctx.h"

#include <stdlib.h>
#include <string.h>

bool __lm_ctx_resolve(lm_ctx_t *ctx, lm_ctx_resolve_list_t *list, lm_pkg_t *pkg){
  if(pkglist_contains(list->packages, pkg))
    return true;

  if(NULL == pkg->depends)
    goto end;

  list->resolving = pkglist_add_top(list->resolving, pkg);
  lm_pkg_t *depend = NULL;

  for(int i = 0; pkg->depends[i] != NULL; i++){
    // if found then its already installed
    if(lm_ctx_database_find(ctx, NULL, pkg->depends[i], NULL))
      continue;

    depend = lm_ctx_pool_find(ctx, pkg->depends[i], NULL);

    if(NULL == depend){
      lm_error_set(LM_ERR_DependNotFound, pkg->depends[i], pkg->name);
      return false;
    }

    if(pkglist_contains(list->resolving, depend)){
      lm_error_set(LM_ERR_DependNotFound, pkg->depends[i], pkg->name);
      return false;
    }

    if(!__lm_ctx_resolve(ctx, list, depend))
      return false;
  }

  list->resolving = pkglist_del(list->resolving, pkg);
end:
  list->packages = pkglist_add_end(list->packages, pkg);
  list->count++;
  return true;
}

lm_ctx_resolve_list_t *lm_ctx_resolve(lm_ctx_t *ctx, lm_pkg_t *pkg){
  if(NULL == ctx || NULL == pkg){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  lm_ctx_resolve_list_t *list = malloc(sizeof(lm_ctx_resolve_list_t));
  bzero(list, sizeof(lm_ctx_resolve_list_t));

  if(!__lm_ctx_resolve(ctx, list, pkg)){
    pkglist_free(list->resolving);
    pkglist_free(list->packages);
    free(list);
    return NULL;
  }

  pkglist_free(list->resolving);
  list->resolving = NULL;

  return list;
}

lm_pkg_t *lm_ctx_resolve_next(lm_ctx_resolve_list_t *list){
  if(NULL == list){
    lm_error_set(LM_ERR_ArgNULL);
    return NULL;
  }

  if(NULL == list->cur){
    list->cur = list->packages;
    return list->cur;
  }

  list->cur = list->cur->next;
  return list->cur;
}

void lm_ctx_resolve_free(lm_ctx_resolve_list_t *list){
  if(NULL == list){
    lm_error_set(LM_ERR_ArgNULL);
    return;
  }

  pkglist_free(list->packages);
  free(list);
  return;
}
